using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Drawing;
using System.Drawing.Design;
using System.Web.UI.Design;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("ClientContextManagerID")]
    [Designer(typeof(ActionImageButtonDesigner))]
    [ToolboxBitmap(typeof(ActionImageButton))]
    [ToolboxData(@"<{0}:ActionImageButton runat=""server""></{0}:ActionImageButton>")]
    public class ActionImageButton : ImageButton, IActionButton
    {
        private const string DefaultValue_ClientContextManagerID = "";
        private const string DefaultValue_EventAction = "";
        private const string DefaultValue_RedirectUrl = "";

        [DefaultValue(DefaultValue_ClientContextManagerID)]
        [SmartDescription("DescriptionAttribute_ActionButton_ClientContextManagerID")]
        [SmartCategory("CategoryAttribute_ActionButton_ClientContextManagerID")]
        [IDReferenceProperty(typeof(ClientContextManager))]
        [TypeConverter(typeof(ClientContextManagerConverter))]
        public string ClientContextManagerID
        {
            get
            {
                Object oObj = ViewState["ClientContextManagerID"];

                if (oObj == null)
                {
                    ViewState["ClientContextManagerID"] = DefaultValue_ClientContextManagerID;
                }

                return ViewState["ClientContextManagerID"] + string.Empty;
            }

            set
            {
                ViewState["ClientContextManagerID"] = value;
            }
        }

        [DefaultValue(DefaultValue_EventAction)]
        [SmartDescription("DescriptionAttribute_ActionButton_EventAction")]
        [SmartCategory("CategoryAttribute_ActionButton_EventAction")]
        [Editor(typeof(EventActionEditor), typeof(UITypeEditor))]
        public string EventAction
        {
            get
            {
                Object oObj = ViewState["EventAction"];

                if (oObj == null)
                {
                    ViewState["EventAction"] = DefaultValue_EventAction;
                }

                return ViewState["EventAction"] + string.Empty;
            }

            set
            {
                ViewState["EventAction"] = value;
            }
        }

        [DefaultValue(DefaultValue_RedirectUrl)]
        [SmartDescription("DescriptionAttribute_ActionButton_RedirectUrl")]
        [SmartCategory("CategoryAttribute_ActionButton_RedirectUrl")]
        [Editor(typeof(UrlEditor), typeof(UITypeEditor))]
        public string RedirectUrl
        {
            get
            {
                Object oObj = ViewState["RedirectUrl"];

                if (oObj == null)
                {
                    ViewState["RedirectUrl"] = DefaultValue_RedirectUrl;
                }

                return ViewState["RedirectUrl"] + string.Empty;
            }

            set
            {
                ViewState["RedirectUrl"] = value;
            }
        }

        protected override void OnClick(ImageClickEventArgs e)
        {
            ClientContextManager clientContextManager = null;

            try
            {
                base.OnClick(e);

                clientContextManager = Utility.FindClientContextManager(this, this.ClientContextManagerID);

                if (this.EventAction.Length != 0)
                {
                    if (clientContextManager.Actions.Contains(this.EventAction))
                    {
                        clientContextManager.CompleteProcessInstance(this.EventAction);

                        if (this.RedirectUrl.Length != 0)
                        {
                            this.Context.Response.Redirect(this.RedirectUrl);
                        }
                    }
                    else
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_ActionNotFound, new object[] { this.EventAction }));
                    }
                }
                else
                {
                    throw new HttpException(string.Format(Resources.Exception_ActionIsBlank, new object[] { this.ID }));
                }
            }
            catch
            {
                throw;
            }
        }
    }
}
