using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Drawing;
using System.Drawing.Design;
using System.Web.UI.Design;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("ClientContextManagerID")]
    [Designer(typeof(CompleteButtonDesigner))]
    [ToolboxBitmap(typeof(CompleteButton))]
    [ToolboxData(@"<{0}:CompleteButton runat=""server"" Text=""CompleteButton""></{0}:CompleteButton>")]
    public class CompleteButton : Button, IClientButton
    {
        private const string DefaultValue_ClientContextManagerID = "";
        private const string DefaultValue_ActionsListControlID = "";
        private const string DefaultValue_RedirectUrl = "";

        [DefaultValue(DefaultValue_ClientContextManagerID)]
        [SmartDescription("DescriptionAttribute_CompleteButton_ClientContextManagerID")]
        [SmartCategory("CategoryAttribute_CompleteButton_ClientContextManagerID")]
        [IDReferenceProperty(typeof(ClientContextManager))]
        [TypeConverter(typeof(ClientContextManagerConverter))]
        public string ClientContextManagerID
        {
            get
            {
                Object oObj = ViewState["ClientContextManagerID"];

                if (oObj == null)
                {
                    ViewState["ClientContextManagerID"] = DefaultValue_ClientContextManagerID;
                }

                return ViewState["ClientContextManagerID"] + string.Empty;
            }

            set
            {
                ViewState["ClientContextManagerID"] = value;
            }
        }

        [DefaultValue("")]
        [SmartDescription("DescriptionAttribute_CompleteButton_ActionsListControlID")]
        [SmartCategory("CategoryAttribute_CompleteButton_ActionsListControlID")]
        [IDReferenceProperty(typeof(WebControl))]
        [TypeConverter(typeof(ActionsListControlConverter))]
        public string ActionsListControlID
        {
            get
            {
                Object oObj = ViewState["ActionsListControlID"];

                if (oObj == null)
                {
                    ViewState["ActionsListControlID"] = DefaultValue_ActionsListControlID;
                }

                return ViewState["ActionsListControlID"] + string.Empty;
            }

            set
            {
                ViewState["ActionsListControlID"] = value;
            }
        }

        [DefaultValue(DefaultValue_RedirectUrl)]
        [SmartDescription("DescriptionAttribute_CompleteButton_RedirectUrl")]
        [SmartCategory("CategoryAttribute_CompleteButton_RedirectUrl")]
        [Editor(typeof(UrlEditor), typeof(UITypeEditor))]        
        public string RedirectUrl
        {
            get
            {
                Object oObj = ViewState["RedirectUrl"];

                if (oObj == null)
                {
                    ViewState["RedirectUrl"] = DefaultValue_RedirectUrl;
                }

                return ViewState["RedirectUrl"] + string.Empty;
            }

            set
            {
                ViewState["RedirectUrl"] = value;
            }
        }

        protected override void OnClick(EventArgs e)
        {
            ClientContextManager clientContextManager = null;
            IActionList actionsListControl = null;

            try
            {
                base.OnClick(e);

                clientContextManager = Utility.FindClientContextManager(this, this.ClientContextManagerID);
               
                actionsListControl = Utility.FindActionsListControl(this, this.ActionsListControlID);

                clientContextManager.CompleteProcessInstance(actionsListControl.GetSelectedAction());

                if (this.RedirectUrl.Length != 0)
                {
                    this.Context.Response.Redirect(this.RedirectUrl);
                }
            }
            catch
            {
                throw;
            }
        }
    }
}
