using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Web.UI.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Collections.Specialized;

using SourceCode.Workflow.Client;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("Host")]
    [Designer(typeof(ClientContextManagerDesigner))]
    [ToolboxBitmap(typeof(ClientContextManager))]
    [ToolboxData(@"<{0}:ClientContextManager runat=""server""></{0}:ClientContextManager>")]
    public sealed class ClientContextManager : WorkflowContextManager
    {
        private const string Const_SerialNumberQueryStringKey = "SN";
        private const string Const_Platform = "ASP";

        public const string DefaultValue_Activity = "";
        public const bool DefaultValue_AllocateSlot = true;
        public const bool DefaultValue_IgnoreStatus = false;

        [DefaultValue(DefaultValue_Activity)]
        [SmartDescription("DescriptionAttribute_ClientContextManager_Activity")]
        [SmartCategory("CategoryAttribute_ClientContextManager_Activity")]
        [Editor(typeof(ActivityEditor), typeof(UITypeEditor))]
        public string Activity
        {
            get
            {
                Object oObj = ViewState["Activity"];

                if (oObj == null)
                {
                    ViewState["Activity"] = DefaultValue_Activity;
                }

                return ViewState["Activity"] + string.Empty;
            }

            set
            {
                ViewState["Activity"] = value;
            }
        }

        [DefaultValue(DefaultValue_AllocateSlot)]
        [SmartDescription("DescriptionAttribute_ClientContextManager_AllocateSlot")]
        [SmartCategory("CategoryAttribute_ClientContextManager_AllocateSlot")]
        public bool AllocateSlot
        {
            get
            {
                object oObj = ViewState["AllocateSlot"];

                if (oObj == null)
                {
                    ViewState["AllocateSlot"] = DefaultValue_AllocateSlot;
                }

                return Convert.ToBoolean(ViewState["AllocateSlot"]);
            }

            set
            {
                ViewState["AllocateSlot"] = value;
            }
        }

        [DefaultValue(DefaultValue_IgnoreStatus)]
        [SmartDescription("DescriptionAttribute_ClientContextManager_IgnoreStatus")]
        [SmartCategory("CategoryAttribute_ClientContextManager_IgnoreStatus")]
        public bool IgnoreStatus
        {
            get
            {
                object oObj = ViewState["IgnoreStatus"];

                if (oObj == null)
                {
                    ViewState["IgnoreStatus"] = DefaultValue_IgnoreStatus;
                }

                return Convert.ToBoolean(ViewState["IgnoreStatus"]);
            }

            set
            {
                ViewState["IgnoreStatus"] = value;
            }
        }

        internal int ProcessInstanceID
        {
            get
            {
                object oObj = ViewState["ProcessInstanceID"];

                if (oObj == null)
                {
                    ViewState["ProcessInstanceID"] = 0;
                }

                return Convert.ToInt32(ViewState["ProcessInstanceID"]);
            }

            set
            {
                ViewState["ProcessInstanceID"] = value;
            }
        }

        internal int ActivityInstanceID
        {
            get
            {
                object oObj = ViewState["ActivityInstanceID"];

                if (oObj == null)
                {
                    ViewState["ActivityInstanceID"] = 0;
                }

                return Convert.ToInt32(ViewState["ActivityInstanceID"]);
            }

            set
            {
                ViewState["ActivityInstanceID"] = value;
            }
        }

        internal int ActivityInstanceDestinationID
        {
            get
            {
                object oObj = ViewState["ActivityInstanceDestinationID"];

                if (oObj == null)
                {
                    ViewState["ActivityInstanceDestinationID"] = 0;
                }

                return Convert.ToInt32(ViewState["ActivityInstanceDestinationID"]);
            }

            set
            {
                ViewState["ActivityInstanceDestinationID"] = value;
            }
        }

        internal int EventInstanceID
        {
            get
            {
                object oObj = ViewState["EventInstanceID"];

                if (oObj == null)
                {
                    ViewState["EventInstanceID"] = 0;
                }

                return Convert.ToInt32(ViewState["EventInstanceID"]);
            }

            set
            {
                ViewState["EventInstanceID"] = value;
            }
        }
        
        internal StringCollection Actions
        {
            get
            {
                Object oObj = ViewState["Actions"];

                if (oObj == null)
                {
                    ViewState["Actions"] = new StringCollection();
                }

                return (StringCollection)ViewState["Actions"];
            }

            set
            {
                ViewState["Actions"] = value;
            }
        }

        internal NameValueCollection ActivityInstanceDestinationDataFields
        {
            get
            {
                Object oObj = ViewState["ActivityInstanceDestinationDataFields"];

                if (oObj == null)
                {
                    ViewState["ActivityInstanceDestinationDataFields"] = new NameValueCollection();
                }

                return (NameValueCollection)ViewState["ActivityInstanceDestinationDataFields"];
            }

            set
            {
                ViewState["ActivityInstanceDestinationDataFields"] = value;
            }
        }

        internal NameValueCollection ActivityInstanceDestinationXmlFields
        {
            get
            {
                Object oObj = ViewState["ActivityInstanceDestinationXmlFields"];

                if (oObj == null)
                {
                    ViewState["ActivityInstanceDestinationXmlFields"] = new NameValueCollection();
                }

                return (NameValueCollection)ViewState["ActivityInstanceDestinationXmlFields"];
            }

            set
            {
                ViewState["ActivityInstanceDestinationXmlFields"] = value;
            }
        }

        private void LoadActivityInstanceDestinationFields(ActivityInstanceDestination activityInstanceDestination)
        {
            foreach (DataField activityInstanceDestinationDataField in activityInstanceDestination.DataFields)
            {
                this.ActivityInstanceDestinationDataFields[activityInstanceDestinationDataField.Name] = activityInstanceDestinationDataField.Value + string.Empty;
            }

            foreach (XmlField activityInstanceDestinationXmlField in activityInstanceDestination.XmlFields)
            {
                this.ActivityInstanceDestinationXmlFields[activityInstanceDestinationXmlField.Name] = activityInstanceDestinationXmlField.Value;
            }
        }

        private void UpdateActivityInstanceDestinationFields(ActivityInstanceDestination activityInstanceDestination)
        {
            List<Control> lstControls = new List<Control>();
            IField field = null;

            lstControls = SmartField.GetNonReadOnlyFields(this);

            foreach (Control control in lstControls)
            {
                field = (IField)control;

                if (control != null && field != null && field.Scope == FieldScope.Activity)
                {
                    if (field.Type == FieldType.DataField)
                    {
                        activityInstanceDestination.DataFields[field.Field].Value = field.CurrentValue;
                    }
                    else
                    {
                        activityInstanceDestination.XmlFields[field.Field].Value = Convert.ToString(field.CurrentValue);
                    }
                }
            }
        }  

        protected internal override void VerifyInitializationArguments()
        {
            if (this.Page.Request.QueryString[Const_SerialNumberQueryStringKey] == null)
            {
                throw new HttpException(Resources.Exception_SerialNumberNotPresent);
            }

 	        base.VerifyInitializationArguments();

            if (this.Activity.Length == 0)
            {
                throw new HttpException(string.Format(Resources.Exception_ActivityIsBlank, new object[] { this.ID }));
            }                               
        }

        private void VerifyWorklistItemArguments(WorklistItem worklistItem)
        {
            if (worklistItem.ProcessInstance.FullName != this.Process)
            {
                throw new HttpException(string.Format(Resources.Exception_ProcessMismatch, new object[] { this.Process }));
            }

            if (worklistItem.ActivityInstanceDestination.Name != this.Activity)
            {
                throw new HttpException(string.Format(Resources.Exception_ActivityMismatch, new object[] { this.Activity }));
            }
        }
         
        public void CompleteProcessInstance(string strSelectedAction)
        {
            Connection connection = new Connection();
            WorklistItem worklistItem = null;

            try
            {
                this.VerifyInitializationArguments();

                connection.Open(this.Host);
                
                worklistItem = connection.OpenWorklistItem(this.Page.Request.QueryString[Const_SerialNumberQueryStringKey], Const_Platform, this.AllocateSlot, this.IgnoreStatus);

                this.VerifyWorklistItemArguments(worklistItem);

                base.UpdateProcessInstanceFields(worklistItem.ProcessInstance);
                this.UpdateActivityInstanceDestinationFields(worklistItem.ActivityInstanceDestination);

                worklistItem.Actions[strSelectedAction].Execute();
            }
            catch
            {
                throw;
            }
            finally
            {
                connection.Close();
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            Connection connection = new Connection();
            WorklistItem worklistItem = null;

            try
            {
                base.OnLoad(e);
                                
                this.VerifyInitializationArguments();

                connection.Open(this.Host);

                worklistItem = connection.OpenWorklistItem(this.Page.Request.QueryString["SN"], "ASP", this.AllocateSlot, this.IgnoreStatus);

                this.VerifyWorklistItemArguments(worklistItem);

                this.ProcessInstanceID = worklistItem.ProcessInstance.ID;
                this.ActivityInstanceDestinationID = worklistItem.ActivityInstanceDestination.ID;
                this.ActivityInstanceID = worklistItem.ActivityInstanceDestination.ActInstID;
                this.EventInstanceID = worklistItem.EventInstance.ID;

                base.LoadProcessInstanceFields(worklistItem.ProcessInstance);
                this.LoadActivityInstanceDestinationFields(worklistItem.ActivityInstanceDestination);

                foreach (SourceCode.Workflow.Client.Action action in worklistItem.Actions)
                {
                    this.Actions.Add(action.Name);
                }
            }
            catch
            {
                throw;
            }
            finally
            {
                connection.Close();
            }               
        }
    }
}
