using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Web.UI.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Collections.Specialized;

using SourceCode.Workflow.Client;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{           
    [DefaultProperty("Host")]
    [DefaultEvent("FolioInitializing")]
    [Designer(typeof(StartContextManagerDesigner))]
    [ToolboxBitmap(typeof(StartContextManager))]
    [ToolboxData(@"<{0}:StartContextManager runat=""server""></{0}:StartContextManager>")]
    public sealed class StartContextManager : WorkflowContextManager
    {
        public const string Const_DefaultFolioFormat = "{0} - {1}";

        public const bool DefaultValue_Synchronous = false;

        public delegate void FolioInitializingEventHandler(object sender, FolioInitializingEventArgs e);

        [DefaultValue(DefaultValue_Synchronous)]
        [SmartDescription("DescriptionAttribute_StartContextManager_Synchronous")]
        [SmartCategory("CategoryAttribute_StartContextManager_Synchronous")]
        public bool Synchronous
        {
            get
            {
                Object oObj = ViewState["Synchronous"];

                if (oObj == null)
                {
                    ViewState["Synchronous"] = DefaultValue_Synchronous;
                }

                return Convert.ToBoolean(ViewState["Synchronous"]);
            }

            set
            {
                ViewState["Synchronous"] = value;
            }
        }

        [SmartDescription("DescriptionAttribute_StartContextManager_FolioInitializingEventHandler")]
        [SmartCategory("CategoryAttribute_StartContextManager_FolioInitializingEventHandler")]
        public event FolioInitializingEventHandler FolioInitializing = null;

        private string InitializeFolio()
        { 
            FolioInitializingEventArgs folioInitializingEventArgs = new FolioInitializingEventArgs();

            if (FolioInitializing == null)
            {
                return string.Format(Const_DefaultFolioFormat, this.Process, DateTime.Now);
            }
            else
            {
                FolioInitializing(this, folioInitializingEventArgs);

                return folioInitializingEventArgs.Folio;
            }
        }

        public void StartProcessInstance()
        {
            Connection connection = new Connection();
            ProcessInstance processInstance = null;

            try
            {
                base.VerifyInitializationArguments();

                connection.Open(this.Host);

                processInstance = connection.CreateProcessInstance(this.Process);

                processInstance.Folio = this.InitializeFolio();

                base.UpdateProcessInstanceFields(processInstance);

                connection.StartProcessInstance(processInstance, this.Synchronous);

                base.LoadProcessInstanceFields(processInstance);
            }
            catch
            {
                throw;
            }
            finally
            {
                connection.Close();
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            Connection connection = new Connection();
            ProcessInstance processInstance = null;

            try
            {
                base.OnLoad(e);

                if (!this.Page.IsPostBack)
                {
                    base.VerifyInitializationArguments();

                    connection.Open(this.Host);

                    processInstance = connection.CreateProcessInstance(this.Process);

                    base.LoadProcessInstanceFields(processInstance);                    
                }
            }
            catch
            {
                throw;
            }
            finally
            {
                connection.Close();
            }
        }
    }
}
