using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Web.UI.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Collections.Specialized;
using System.IO;

using SourceCode.Workflow.Client;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("Host")]
    [Designer(typeof(WorkflowContextManagerDesigner))]
    public abstract class WorkflowContextManager : WebControl
    {
        private const string DefaultValue_Host = "";
        private const string DefaultValue_Process = "";

        [DefaultValue(DefaultValue_Host)]
        [SmartDescription("DescriptionAttribute_WorkflowContextManager_Host")]
        [SmartCategory("CategoryAttribute_WorkflowContextManager_Host")]
        [Editor(typeof(HostEditor), typeof(UITypeEditor))]
        public string Host
        {
            get
            {
                Object oObj = ViewState["Host"];

                if (oObj == null)
                {
                    ViewState["Host"] = DefaultValue_Host;
                }

                return ViewState["Host"] + string.Empty;
            }

            set
            {
                ViewState["Host"] = value;
            }
        }

        [DefaultValue(DefaultValue_Process)]
        [SmartDescription("DescriptionAttribute_WorkflowContextManager_Process")]
        [SmartCategory("CategoryAttribute_WorkflowContextManager_Process")]
        [Editor(typeof(ProcessEditor), typeof(UITypeEditor))]
        public string Process
        {
            get
            {
                Object oObj = ViewState["Process"];

                if (oObj == null)
                {
                    ViewState["Process"] = DefaultValue_Process;
                }

                return ViewState["Process"] + string.Empty;
            }

            set
            {
                ViewState["Process"] = value;                
            }
        }

        protected internal NameValueCollection ProcessInstanceDataFields
        {
            get
            {
                Object oObj = ViewState["ProcessInstanceDataFields"];

                if (oObj == null)
                {
                    ViewState["ProcessInstanceDataFields"] = new NameValueCollection();
                }

                return (NameValueCollection)ViewState["ProcessInstanceDataFields"];
            }

            set
            {
                ViewState["ProcessInstanceDataFields"] = value;
            }
        }

        protected internal NameValueCollection ProcessInstanceXmlFields
        {
            get
            {
                Object oObj = ViewState["ProcessInstanceXmlFields"];

                if (oObj == null)
                {
                    ViewState["ProcessInstanceXmlFields"] = new NameValueCollection();
                }

                return (NameValueCollection)ViewState["ProcessInstanceXmlFields"];
            }

            set
            {
                ViewState["ProcessInstanceXmlFields"] = value;
            }
        }

        protected void LoadProcessInstanceFields(ProcessInstance processInstance)
        {
            foreach (DataField processDataField in processInstance.DataFields)
            {
                this.ProcessInstanceDataFields[processDataField.Name] = processDataField.Value + string.Empty;
            }

            foreach (XmlField processXmlField in processInstance.XmlFields)
            {                
                this.ProcessInstanceXmlFields[processXmlField.Name] = processXmlField.Value;
            }
        }

        protected void UpdateProcessInstanceFields(ProcessInstance processInstance)
        {
            List<Control> lstControls = new List<Control>();
            IField field = null;

            lstControls = SmartField.GetNonReadOnlyFields(this);

            foreach (Control control in lstControls)
            {
                field = (IField)control;

                if (control != null && field != null && field.Scope == FieldScope.Process)
                {
                    if (field.Type == FieldType.DataField)
                    {
                        processInstance.DataFields[field.Field].Value = field.CurrentValue;
                    }
                    else
                    {
                        processInstance.XmlFields[field.Field].Value = Convert.ToString(field.CurrentValue);
                    }
                }
            }
        }

        protected internal virtual void VerifyInitializationArguments()
        {
            if (this.Host.Length == 0)
            {
                throw new HttpException(string.Format(Resources.Exception_HostIsBlank, new object[] { this.ID }));
            }

            if (this.Process.Length == 0)
            {
                throw new HttpException(string.Format(Resources.Exception_ProcessIsBlank, new object[] { this.ID }));
            }

            if (this.Process.IndexOf(Path.DirectorySeparatorChar) <= 0)
            {
                throw new FormatException(string.Format(Resources.Exception_IncorrectProcessFormat, new object[] { this.Process }));
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
        }
    }
}