using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Web;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls.Design
{
    partial class ActivityPickerForm : EditorForm
    {
        private ClientContextManager _clientContextManager = null;

        public override object Value
        {
            get
            {
                return activitiesTreeView.SelectedNode.Tag;
            }
        }

        protected override bool IsValid
        {
            get
            {
                return (activitiesTreeView.SelectedNode != null && activitiesTreeView.SelectedNode.Tag is Activity && activitiesTreeView.SelectedNode.Level == 1);
            }
        }

        public ActivityPickerForm(ClientContextManager clientContextManager)
        {            
            try
            {
                if (clientContextManager.Host.Length != 0)
                {
                    if (clientContextManager.Process.Length != 0)
                    {
                        InitializeComponent();

                        _clientContextManager = clientContextManager;

                        LoadActivities(false);
                    }
                    else
                    {
                        throw new HttpException(string.Format(Resources.Exception_ProcessIsBlank, new object[] { clientContextManager.ID }));
                    }
                }
                else
                {
                    throw new HttpException(string.Format(Resources.Exception_HostIsBlank, new object[] { clientContextManager.ID }));
                }
            }
            catch
            {
                throw;
            }
        }

        private void LoadActivities(bool bRefresh)
        {
            WorkflowServerManager workflowServerManager = null;
            ProcessSet processSet = null;
            TreeNode nodeProcessSet = null;
            TreeNode nodeActivity = null;
            Activities activities = null;
            Events evts = null;

            try
            {
                Cursor.Current = Cursors.WaitCursor;

                activitiesTreeView.SuspendLayout();

                activitiesTreeView.Nodes.Clear();

                workflowServerManager = new WorkflowServerManager(_clientContextManager.Host);

                workflowServerManager.OpenConnection();

                processSet = workflowServerManager.GetProcessSet(_clientContextManager.Process);

                nodeProcessSet = new TreeNode(processSet.Name, processSet.Description, processSet);

                activities = workflowServerManager.GetProcActivities(processSet.ProcID);

                foreach (Activity activity in activities)
                {
                    evts = workflowServerManager.GetActivityEvents(activity.ID);

                    foreach (Event evt in evts)
                    {
                        if (evt.EventType == EventTypes.ClientEvent)
                        {
                            nodeActivity = new TreeNode(activity.Name, activity.Description, activity);

                            nodeProcessSet.Nodes.Add(nodeActivity);

                            if (!bRefresh)
                            {
                                if (_clientContextManager.Activity == activity.Name)
                                {
                                    activitiesTreeView.SelectedNode = nodeActivity;
                                }
                            }

                            break;
                        }
                    }
                }

                activitiesTreeView.Nodes.Add(nodeProcessSet);

                nodeProcessSet.Expand();
            }
            catch
            {
                throw;
            }
            finally
            {
                if (workflowServerManager != null)
                {
                    workflowServerManager.CloseConnection();
                }

                activitiesTreeView.ResumeLayout();

                Cursor.Current = Cursors.Default;
            }
        }

        private void activitiesTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            okButton.Enabled = this.IsValid;
        }

        private void activitiesTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (this.IsValid)
            {
                this.DialogResult = DialogResult.OK;

                this.Close();
            }
        }

        private void refreshLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LoadActivities(true);
            }
            catch (Exception exp)
            {
                this.HandleError(exp);
            }
            finally
            {
                okButton.Enabled = this.IsValid;
            }
        }
    }
}