using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Web;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls.Design
{
    partial class EventActionPickerForm : EditorForm
    {
        private ClientContextManager _clientContextManager = null;
        private System.Web.UI.Control _control = null;
        private IActionButton _actionButton = null;

        public override object Value
        {
            get
            {
                return actionsTreeView.SelectedNode.Tag;
            }
        }

        protected override bool IsValid
        {
            get
            {
                return (actionsTreeView.SelectedNode != null && actionsTreeView.SelectedNode.Tag is EventAction && actionsTreeView.SelectedNode.Level == 1);
            }
        }

        public EventActionPickerForm(System.Web.UI.Control control)
        {
            try
            {
                _control = control;
                _actionButton = (IActionButton)control;
                _clientContextManager = Utility.FindClientContextManager(_control, _actionButton.ClientContextManagerID);

                if (_clientContextManager.Host.Length != 0)
                {
                    if (_clientContextManager.Process.Length != 0)
                    {
                        if (_clientContextManager.Activity.Length != 0)
                        {
                            InitializeComponent();

                            LoadActions(false);

                            okButton.Enabled = this.IsValid;
                        }
                        else
                        {
                            throw new HttpException(string.Format(Resources.Exception_ActivityIsBlank, new object[] { _clientContextManager.ID }));
                        }
                    }
                    else
                    {
                        throw new HttpException(string.Format(Resources.Exception_ProcessIsBlank, new object[] { _clientContextManager.ID }));
                    }
                }
                else
                {
                    throw new HttpException(string.Format(Resources.Exception_HostIsBlank, new object[] { _clientContextManager.ID }));
                }
            }
            catch
            {
                throw;
            }
        }

        private void LoadActions(bool bRefresh)
        {
            WorkflowServerManager workflowServerManager = null;
            TreeNode nodeActivity = null;
            TreeNode nodeEventAction = null;
            Activity activity = null;
            Events evts = null;
            EventActions eventActions = null;
            
            try
            {
                Cursor.Current = Cursors.WaitCursor;

                actionsTreeView.SuspendLayout();

                actionsTreeView.Nodes.Clear();

                workflowServerManager = new WorkflowServerManager(_clientContextManager.Host);

                workflowServerManager.OpenConnection();

                activity = workflowServerManager.GetActivity(_clientContextManager.Process, _clientContextManager.Activity);

                nodeActivity = new TreeNode(activity.Name, activity.Description, activity);
                
                evts = workflowServerManager.GetActivityEvents(activity.ID);

                foreach (Event evt in evts)
                {
                    if (evt.EventType == EventTypes.ClientEvent)
                    {
                        eventActions = workflowServerManager.GetEventActions(evt.ID);

                        foreach (EventAction eventAction in eventActions)
                        {
                            nodeEventAction = new TreeNode(eventAction.Name, eventAction.MetaData, eventAction);

                            nodeActivity.Nodes.Add(nodeEventAction);

                            if (!bRefresh)
                            {
                                if (_actionButton.EventAction == eventAction.Name)
                                {
                                    actionsTreeView.SelectedNode = nodeEventAction;
                                }
                            }
                        }
                                
                        break;
                    }
                }

                actionsTreeView.Nodes.Add(nodeActivity);

                nodeActivity.Expand();
            }
            catch
            {
                throw;
            }
            finally
            {
                if (workflowServerManager != null)
                {
                    workflowServerManager.CloseConnection();
                }

                actionsTreeView.ResumeLayout();

                Cursor.Current = Cursors.Default;
            }
        }

        private void activitiesTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            okButton.Enabled = this.IsValid;
        }

        private void activitiesTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (this.IsValid)
            {
                this.DialogResult = DialogResult.OK;

                this.Close();
            }
        }

        private void refreshLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LoadActions(true);
            }
            catch (Exception exp)
            {
                this.HandleError(exp);
            }
            finally
            {
                okButton.Enabled = this.IsValid;
            }
        }
    }
}