using System;
using System.Collections.Generic;
using System.Text;

using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Web.UI.WebControls;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls.Design
{
    class FieldEditor : Editor
    {
        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext typeDescriptorContext)
        {
            return UITypeEditorEditStyle.Modal;
        }        

        public override object EditValue(ITypeDescriptorContext typeDescriptorContext, IServiceProvider serviceProvider, object oValue)
        {
            IWindowsFormsEditorService windowsFormsEditorService = null;
            FieldPickerForm fieldPicker = null;
            System.Web.UI.Control control = null;
            IField field = null;
            ProcessDataField processDataField = null;
            ProcessXMLField processXmlField = null;
            ActivityDataField activityDataField = null;
            ActivityXMLField activityXmlField = null;
            PropertyManager propertyManager = null;
            ActionList actionList = null;

            try
            {
                if (typeDescriptorContext.Instance != null && serviceProvider != null)
                {
                    windowsFormsEditorService = (IWindowsFormsEditorService)serviceProvider.GetService(typeof(IWindowsFormsEditorService));

                    if (windowsFormsEditorService != null)
                    {
                        if (typeDescriptorContext.Instance is ActionList)
                        {
                            actionList = (ActionList)typeDescriptorContext.Instance;

                            control = (System.Web.UI.Control)actionList.Component;
                        }
                        else
                        {
                            control = (System.Web.UI.Control)typeDescriptorContext.Instance;
                        }

                        field = (IField)control;

                        fieldPicker = new FieldPickerForm(control);

                        if (windowsFormsEditorService.ShowDialog(fieldPicker) == DialogResult.OK)
                        {
                            propertyManager = new PropertyManager(control);
                            
                            if (fieldPicker.Value is ProcessDataField)
                            {
                                processDataField = (ProcessDataField)fieldPicker.Value;
                                oValue = processDataField.Name;
                                propertyManager.SetPropertyValue("Type", FieldType.DataField);
                                propertyManager.SetPropertyValue("Scope", FieldScope.Process);
                            }
                            else if (fieldPicker.Value is ProcessXMLField)
                            {
                                processXmlField = (ProcessXMLField)fieldPicker.Value;
                                oValue = processXmlField.Name;
                                propertyManager.SetPropertyValue("Type", FieldType.XmlField);
                                propertyManager.SetPropertyValue("Scope", FieldScope.Process);
                            }
                            else if( fieldPicker.Value is ActivityDataField)
                            {
                                activityDataField = (ActivityDataField)fieldPicker.Value;
                                oValue = activityDataField.Name;
                                propertyManager.SetPropertyValue("Type", FieldType.DataField);
                                propertyManager.SetPropertyValue("Scope", FieldScope.Activity);
                            }
                            else if (fieldPicker.Value is ActivityXMLField)
                            {
                                activityXmlField = (ActivityXMLField)fieldPicker.Value;
                                oValue = activityXmlField.Name;
                                propertyManager.SetPropertyValue("Type", FieldType.XmlField);
                                propertyManager.SetPropertyValue("Scope", FieldScope.Activity);
                            }
                        }
                    }
                }
            }
            catch(Exception exp)
            {
                this.DisplayError(serviceProvider, exp);
            }

            return oValue;
        }
    }
}
