using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Web;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls.Design
{
    enum FieldTypeState
    {
        All, DataFields, XmlFields
    }

    enum FieldScopeState
    {
        All, Process, Activity
    }

    partial class FieldPickerForm : EditorForm
    {
        private string strDataFieldsNodeText = string.Empty;
        private string strXmlFieldsNodeText = string.Empty;
        private FieldTypeState _fieldTypeState = FieldTypeState.All;
        private FieldScopeState _fieldScopeState = FieldScopeState.All;
        private WorkflowContextManager _workflowContextManager = null;
        private StartContextManager _startContextManager = null;
        private ClientContextManager _clientContextManager = null;
        private System.Web.UI.Control _control = null;
        private IField _field = null;

        public override object Value
        {
            get
            {
                return fieldsTreeView.SelectedNode.Tag;
            }
        }

        protected override bool IsValid
        {
            get
            {
                bool bIsField = false;

                bIsField = (fieldsTreeView.SelectedNode != null && (fieldsTreeView.SelectedNode.Tag is ProcessDataField || fieldsTreeView.SelectedNode.Tag is ProcessXMLField || fieldsTreeView.SelectedNode.Tag is ActivityDataField || fieldsTreeView.SelectedNode.Tag is ActivityXMLField));

                return (bIsField && fieldsTreeView.SelectedNode.Level == 2);
            }
        }

        public FieldPickerForm(System.Web.UI.Control control)
        {
            ComponentResourceManager componentResourceManager = null;

            try
            {
                _control = control;
                _field = (IField)control; 
                _workflowContextManager = Utility.FindWorkflowContextManager(_control, _field.WorkflowContextManagerID);
                
                _fieldTypeState = DetermineFieldTypeState(control);
                _fieldScopeState = DetermineFieldScopeState(_control);

                if (_workflowContextManager.Host.Length != 0)
                {
                    if (_workflowContextManager.Process.Length != 0)
                    {
                        if (_workflowContextManager is StartContextManager)
                        {
                            _startContextManager = (StartContextManager)_workflowContextManager;
                        }
                        else
                        {
                            _clientContextManager = (ClientContextManager)_workflowContextManager;

                            if (_clientContextManager.Activity.Length == 0)
                            {
                                throw new HttpException(string.Format(Resources.Exception_ActivityIsBlank, new object[] { _workflowContextManager.ID }));
                            }
                        }

                        InitializeComponent();

                        componentResourceManager = new ComponentResourceManager(typeof(FieldPickerForm));

                        strDataFieldsNodeText = componentResourceManager.GetObject("dataFieldsTreeNode.Text") + string.Empty;
                        strXmlFieldsNodeText = componentResourceManager.GetObject("xmlFieldsTreeNode.Text") + string.Empty;

                        LoadFields(false);

                        okButton.Enabled = this.IsValid;
                    }
                    else
                    {
                        throw new HttpException(string.Format(Resources.Exception_ProcessIsBlank, new object[] { _workflowContextManager.ID }));
                    }
                }
                else
                {
                    throw new HttpException(string.Format(Resources.Exception_HostIsBlank, new object[] { _workflowContextManager.ID }));
                }
            }
            catch
            {                
                throw;
            }
        }

        private FieldTypeState DetermineFieldTypeState(System.Web.UI.Control control)
        {
            PropertyManager propertyManager = null;
            FieldType fieldType = FieldType.DataField;
            FieldTypeState fieldTypeState = FieldTypeState.All;

            try
            {
                propertyManager = new PropertyManager(control);

                if (propertyManager.IsPropertyReadOnly("Type"))
                {
                    fieldType = (FieldType)propertyManager.GetPropertyValue("Type");

                    if (fieldType == FieldType.DataField)
                    {
                        fieldTypeState = FieldTypeState.DataFields;
                    }
                    else
                    {
                        fieldTypeState = FieldTypeState.XmlFields;
                    }
                }
                else
                {
                    fieldTypeState = FieldTypeState.All;
                }
            }
            catch
            {
                throw;
            }

            return fieldTypeState;
        }

        private FieldScopeState DetermineFieldScopeState(System.Web.UI.Control control)
        {
            PropertyManager propertyManager = null;
            FieldScope fieldScope = FieldScope.Process;
            FieldScopeState fieldScopeState = FieldScopeState.All;

            try
            {
                propertyManager = new PropertyManager(control);

                if (propertyManager.IsPropertyReadOnly("Scope"))
                {
                    fieldScope = (FieldScope)propertyManager.GetPropertyValue("Scope");

                    if (fieldScope == FieldScope.Process)
                    {
                        fieldScopeState = FieldScopeState.Process;
                    }
                    else
                    {
                        fieldScopeState = FieldScopeState.Activity;
                    }
                }
                else
                {
                    fieldScopeState = FieldScopeState.All;
                }
            }
            catch
            {
                throw;
            }

            return fieldScopeState;
        }

        private TreeNode BuildProcessDataFieldsSubTree(ProcessDataFields processDataFields, bool bRefresh)
        {
            TreeNode nodeProcessDataFields = null;
            TreeNode nodeProcessDataField = null;

            nodeProcessDataFields = new TreeNode(strDataFieldsNodeText, string.Empty, processDataFields);

            foreach (ProcessDataField processDataField in processDataFields)
            {
                nodeProcessDataField = new TreeNode(processDataField.Name, processDataField.MetaData, processDataField);

                if (!bRefresh)
                {
                    if (_field.Type == FieldType.DataField && _field.Scope == FieldScope.Process && _field.Field == processDataField.Name)
                    {
                        fieldsTreeView.SelectedNode = nodeProcessDataField;

                        nodeProcessDataFields.Expand();
                    }
                }

                nodeProcessDataFields.Nodes.Add(nodeProcessDataField);
            }

            return nodeProcessDataFields;
        }

        private TreeNode BuildProcessXmlFieldsSubTree(ProcessXMLFields processXmlFields, bool bRefresh)
        {
            TreeNode nodeProcessXmlFields = null;
            TreeNode nodeProcessXmlField = null;

            nodeProcessXmlFields = new TreeNode(strXmlFieldsNodeText, string.Empty, processXmlFields);

            foreach (ProcessXMLField processXmlField in processXmlFields)
            {
                nodeProcessXmlField = new TreeNode(processXmlField.Name, processXmlField.MetaData, processXmlField);

                if (!bRefresh)
                {
                    if (_field.Type == FieldType.XmlField && _field.Scope == FieldScope.Process && _field.Field == processXmlField.Name)
                    {
                        fieldsTreeView.SelectedNode = nodeProcessXmlField;

                        nodeProcessXmlFields.Expand();
                    }
                }

                nodeProcessXmlFields.Nodes.Add(nodeProcessXmlField);
            }

            return nodeProcessXmlFields;
        }

        private TreeNode BuildActivityDataFieldsSubTree(ActivityDataFields activityDataFields, bool bRefresh)
        {
            TreeNode nodeActivityDataFields = null;
            TreeNode nodeActivityDataField = null;

            nodeActivityDataFields = new TreeNode(strDataFieldsNodeText, string.Empty, activityDataFields);

            foreach (ActivityDataField activityDataField in activityDataFields)
            {
                nodeActivityDataField = new TreeNode(activityDataField.Name, activityDataField.MetaData, activityDataField);

                if (!bRefresh)
                {
                    if (_field.Type == FieldType.DataField && _field.Scope == FieldScope.Activity && _field.Field == activityDataField.Name)
                    {
                        fieldsTreeView.SelectedNode = nodeActivityDataField;

                        nodeActivityDataFields.Expand();
                    }
                }

                nodeActivityDataFields.Nodes.Add(nodeActivityDataField);
            }

            return nodeActivityDataFields;
        }

        private TreeNode BuildActivityXmlFieldsSubTree(ActivityXMLFields activityXmlFields, bool bRefresh)
        {
            TreeNode nodeActivityXmlFields = null;
            TreeNode nodeActivityXmlField = null;

            nodeActivityXmlFields = new TreeNode(strXmlFieldsNodeText, string.Empty, activityXmlFields);

            foreach (ActivityXMLField activityXmlField in activityXmlFields)
            {
                nodeActivityXmlField = new TreeNode(activityXmlField.Name, activityXmlField.MetaData, activityXmlField);

                if (!bRefresh)
                {
                    if (_field.Type == FieldType.XmlField && _field.Scope == FieldScope.Activity && _field.Field == activityXmlField.Name)
                    {
                        fieldsTreeView.SelectedNode = nodeActivityXmlField;

                        nodeActivityXmlFields.Expand();
                    }
                }

                nodeActivityXmlFields.Nodes.Add(nodeActivityXmlField);
            }

            return nodeActivityXmlFields;
        }

        private void LoadFields(bool bRefresh)
        {
            TreeNode nodeProcessSet = null;
            TreeNode nodeProcessDataFields = null;
            TreeNode nodeProcessXmlFields = null;   
            TreeNode nodeActivity = null;
            TreeNode nodeActivityDataFields = null;
            TreeNode nodeActivityXmlFields = null;
            WorkflowServerManager workflowServerManager = null;
            ProcessSet processSet = null;
            Activity activity = null;
            ProcessDataFields processDataFields = null;
            ProcessXMLFields processXmlFields = null;
            ActivityDataFields activityDataFields = null;
            ActivityXMLFields activityXmlFields = null;

            try
            {
                Cursor.Current = Cursors.WaitCursor;

                fieldsTreeView.SuspendLayout();

                fieldsTreeView.Nodes.Clear();

                workflowServerManager = new WorkflowServerManager(_workflowContextManager.Host);
                
                workflowServerManager.OpenConnection();

                processSet = workflowServerManager.GetProcessSet(_workflowContextManager.Process);

                if (_fieldScopeState == FieldScopeState.All || _fieldScopeState == FieldScopeState.Process)
                {
                    nodeProcessSet = new TreeNode(processSet.Name, processSet.Description, processSet);

                    if (_fieldTypeState == FieldTypeState.All || _fieldTypeState == FieldTypeState.DataFields)
                    {
                        processDataFields = workflowServerManager.GetProcessDataFields(processSet.ProcID);
                        nodeProcessDataFields = BuildProcessDataFieldsSubTree(processDataFields, bRefresh);
                        nodeProcessSet.Nodes.Add(nodeProcessDataFields);
                    }

                    if (_fieldTypeState == FieldTypeState.All || _fieldTypeState == FieldTypeState.XmlFields)
                    {
                        processXmlFields = workflowServerManager.GetProcessXMLFields(processSet.ProcID);
                        nodeProcessXmlFields = BuildProcessXmlFieldsSubTree(processXmlFields, bRefresh);
                        nodeProcessSet.Nodes.Add(nodeProcessXmlFields);
                    }

                    fieldsTreeView.Nodes.Add(nodeProcessSet);

                    nodeProcessSet.Expand();
                }

                if (_clientContextManager != null)
                {
                    activity = workflowServerManager.GetActivity(_clientContextManager.Process, _clientContextManager.Activity);

                    if (_fieldScopeState == FieldScopeState.All || _fieldScopeState == FieldScopeState.Activity)
                    {
                        nodeActivity = new TreeNode(activity.Name, activity.Description, activity);

                        if (_fieldTypeState == FieldTypeState.All || _fieldTypeState == FieldTypeState.DataFields)
                        {
                            activityDataFields = workflowServerManager.GetActivityDataFields(processSet.ProcID, activity.ID);
                            nodeActivityDataFields = BuildActivityDataFieldsSubTree(activityDataFields, bRefresh);
                            nodeActivity.Nodes.Add(nodeActivityDataFields);
                        }

                        if (_fieldTypeState == FieldTypeState.All || _fieldTypeState == FieldTypeState.XmlFields)
                        {
                            activityXmlFields = workflowServerManager.GetActivityXmlFields(processSet.ProcID, activity.ID);
                            nodeActivityXmlFields = BuildActivityXmlFieldsSubTree(activityXmlFields, bRefresh);
                            nodeActivity.Nodes.Add(nodeActivityXmlFields);
                        }

                        fieldsTreeView.Nodes.Add(nodeActivity);

                        nodeActivity.Expand();
                    }
                }
            }
            catch
            {
                throw;
            }
            finally
            {
                if (workflowServerManager != null)
                {
                    workflowServerManager.CloseConnection();
                }

                fieldsTreeView.ResumeLayout();

                Cursor.Current = Cursors.Default;
            }
        }
        
        private void fieldsTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            okButton.Enabled = this.IsValid;
        }

        private void fieldsTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (this.IsValid)
            {
                this.DialogResult = DialogResult.OK;

                this.Close();
            }
        }

        private void refreshLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LoadFields(true);
            }
            catch (Exception exp)
            {
                this.HandleError(exp);
            }
            finally
            {
                okButton.Enabled = this.IsValid;
            }
        }
    }
}