using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Web;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls.Design
{
    partial class ProcessPickerForm : EditorForm
    {
        private WorkflowContextManager _workflowContextManager = null;
        
        public override object Value
        {
            get
            {
                return processesTreeView.SelectedNode.Tag;
            }
        }

        protected override bool IsValid
        {
            get
            {                
                return (processesTreeView.SelectedNode != null && processesTreeView.SelectedNode.Tag is ProcessSet && processesTreeView.SelectedNode.Level == 2);
            }
        }

        public ProcessPickerForm(WorkflowContextManager workflowContextManager)
        {
            try
            {
                if (workflowContextManager.Host.Length != 0)
                {
                    _workflowContextManager = workflowContextManager;

                    InitializeComponent();

                    LoadProcesses(false);

                    okButton.Enabled = this.IsValid;
                }
                else
                {
                    throw new HttpException(string.Format(Resources.Exception_HostIsBlank, new object[] { workflowContextManager.ID }));
                }
            }
            catch
            {
                throw;
            }
        }

        private void LoadProcesses(bool bRefresh)
        {
            WorkflowServerManager workflowServerManager = null;
            TreeNode nodeConnection = null;
            TreeNode nodeProcessFolder = null;
            TreeNode nodeProcessSet = null;
            ProcessFolders processFolders = null;

            try
            {
                Cursor.Current = Cursors.WaitCursor;

                processesTreeView.SuspendLayout();

                processesTreeView.Nodes.Clear();

                workflowServerManager = new WorkflowServerManager(_workflowContextManager.Host);

                workflowServerManager.OpenConnection();

                nodeConnection = new TreeNode(workflowServerManager.Connection.Host, string.Empty, workflowServerManager.Connection);

                processFolders = workflowServerManager.GetProcessFolders();

                foreach (ProcessFolder processFolder in processFolders)
                {
                    nodeProcessFolder = new TreeNode(processFolder.FolderName, string.Empty, processFolder);

                    foreach (ProcessSet processSet in processFolder.ProcessSets)
                    {
                        nodeProcessSet = new TreeNode(processSet.Name, processSet.Description, processSet);

                        nodeProcessFolder.Nodes.Add(nodeProcessSet);

                        if (!bRefresh)
                        {
                            if (_workflowContextManager.Process == processSet.FullName)
                            {
                                processesTreeView.SelectedNode = nodeProcessSet;

                                nodeProcessFolder.Expand();
                            }
                        }   
                    }

                    nodeConnection.Nodes.Add(nodeProcessFolder);
                }

                processesTreeView.Nodes.Add(nodeConnection);

                nodeConnection.Expand();
                
            }
            catch
            {
                throw;
            }
            finally
            {
                if (workflowServerManager != null)
                {
                    workflowServerManager.CloseConnection();
                }

                processesTreeView.ResumeLayout();

                Cursor.Current = Cursors.Default;
            }
        }

        private void processesTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            okButton.Enabled = this.IsValid;
        }

        private void processesTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (this.IsValid)
            {
                this.DialogResult = DialogResult.OK;

                this.Close();
            }
        }

        private void refreshLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LoadProcesses(true);
            }
            catch (Exception exp)
            {
                this.HandleError(exp);
            }
            finally
            {
                okButton.Enabled = this.IsValid;
            }
        }
    }
}