using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Drawing;
using System.Drawing.Design;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("WorkflowContextManagerID")]
    [Designer(typeof(SmartListControlDesigner))]
    [ToolboxBitmap(typeof(SmartCheckBoxList))]
    [ToolboxData(@"<{0}:SmartCheckBoxList runat=""server""></{0}:SmartCheckBoxList>")]
    public class SmartCheckBoxList : CheckBoxList, IField
    {
        private const string DefaultValue_WorkflowContextManagerID = "";
        private const string DefaultValue_Field = "";
        private const FieldScope DefaultValue_Scope = FieldScope.Process;
        private const FieldType DefaultValue_Type = FieldType.DataField;
        private const bool DefaultValue_PopulateOnLoad = true;
        public const char DefaultValue_ValueSeparator = ';';

        [DefaultValue(DefaultValue_WorkflowContextManagerID)]
        [SmartDescription("DescriptionAttribute_FieldControl_WorkflowContextManagerID")]
        [SmartCategory("CategoryAttribute_FieldControl_WorkflowContextManagerID")]
        [IDReferenceProperty(typeof(WorkflowContextManager))]
        [TypeConverter(typeof(WorkflowContextManagerConverter))]
        public string WorkflowContextManagerID
        {
            get
            {
                Object oObj = ViewState["WorkflowContextManagerID"];

                if (oObj == null)
                {
                    ViewState["WorkflowContextManagerID"] = DefaultValue_WorkflowContextManagerID;
                }

                return ViewState["WorkflowContextManagerID"] + string.Empty;
            }

            set
            {
                ViewState["WorkflowContextManagerID"] = value;
            }
        }

        [DefaultValue(DefaultValue_Field)]
        [SmartDescription("DescriptionAttribute_FieldControl_Field")]
        [SmartCategory("CategoryAttribute_FieldControl_Field")]
        [Editor(typeof(FieldEditor), typeof(UITypeEditor))]
        public string Field
        {
            get
            {
                Object oObj = ViewState["Field"];

                if (oObj == null)
                {
                    ViewState["Field"] = DefaultValue_Field;
                }

                return ViewState["Field"] + string.Empty;
            }

            set
            {
                ViewState["Field"] = value;
            }
        }

        [DefaultValue(DefaultValue_Scope)]
        [SmartDescription("DescriptionAttribute_FieldControl_Scope")]
        [SmartCategory("CategoryAttribute_FieldControl_Scope")]
        public FieldScope Scope
        {
            get
            {
                Object oObj = ViewState["Scope"];

                if (oObj == null)
                {
                    ViewState["Scope"] = DefaultValue_Scope;
                }

                return (FieldScope)ViewState["Scope"];
            }

            set
            {
                ViewState["Scope"] = value;
            }
        }

        [ReadOnly(true)]
        [DefaultValue(DefaultValue_Type)]
        [SmartDescription("DescriptionAttribute_FieldControl_Type")]
        [SmartCategory("CategoryAttribute_FieldControl_Type")]
        public FieldType Type
        {
            get
            {
                return DefaultValue_Type;
            }
        }

        [DefaultValue(DefaultValue_PopulateOnLoad)]
        [SmartDescription("DescriptionAttribute_FieldControl_PopulateOnLoad")]
        [SmartCategory("CategoryAttribute_FieldControl_PopulateOnLoad")]
        public bool PopulateOnLoad
        {
            get
            {
                object oObj = ViewState["PopulateOnLoad"];

                if (oObj == null)
                {
                    ViewState["PopulateOnLoad"] = DefaultValue_PopulateOnLoad;
                }

                return Convert.ToBoolean(ViewState["PopulateOnLoad"]);
            }

            set
            {
                ViewState["PopulateOnLoad"] = value;
            }
        }

        public bool LoadOnly
        {
            get
            {
                return false;
            }
        }

        public string CurrentValue
        {
            get
            {
                return this.SelectedValue;
            }
        }

        [DefaultValue(DefaultValue_ValueSeparator)]
        [SmartDescription("DescriptionAttribute_MultiValueFieldControl_ValueSeparator")]
        [SmartCategory("CategoryAttribute_MultiValueFieldControl_ValueSeparator")]
        public char ValueSeparator
        {
            get
            {
                object oObj = ViewState["Separator"];

                if (oObj == null)
                {
                    ViewState["Separator"] = DefaultValue_ValueSeparator;
                }

                return Convert.ToChar(ViewState["Separator"]);
            }

            set
            {
                ViewState["Separator"] = value;
            }
        }

        public override string SelectedValue
        {
            get
            {
                string selectedValue = string.Empty;

                foreach (ListItem listItem in this.Items)
                {
                    if (listItem.Selected)
                    {
                        selectedValue += (listItem.Value + this.ValueSeparator);
                    }
                }

                return selectedValue;
            }

            set
            {
                string[] selectedValues = null;
                bool matchingListItemFound = false;

                selectedValues = value.Split(new char[] { this.ValueSeparator });

                foreach (ListItem listItem in this.Items)
                {
                    listItem.Selected = false;
                    matchingListItemFound = false;

                    for (int i = 0; i < selectedValues.Length && !matchingListItemFound; i++)
                    {
                        if (listItem.Value == selectedValues[i])
                        {
                            listItem.Selected = true;
                            matchingListItemFound = true;
                        }
                    }
                }
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            try
            {
                base.OnLoad(e);

                if (!this.Page.IsPostBack)
                {
                    if (this.PopulateOnLoad)
                    {
                        this.SelectedValue = Convert.ToString(SmartField.GetCurrentValue(this));
                    }
                }
            }
            catch
            {
                throw;
            }
        }
    }
}
