using System;
using System.Collections.Generic;
using System.Text;

using SourceCode.Hosting.Client.BaseAPI;

namespace Dauphine.Web.UI.SmartControls
{
    class ConnectionStringBuilder
    {
        public static string CreateBaseConnectionString(string strHost, uint nPort)
        {
            SCConnectionStringBuilder connectionStringBuilder = new SCConnectionStringBuilder();

            connectionStringBuilder.Authenticate = true;
            connectionStringBuilder.Host = strHost;
            connectionStringBuilder.Port = nPort;
            connectionStringBuilder.Integrated = true;
            connectionStringBuilder.IsPrimaryLogin = true;

            return connectionStringBuilder.ConnectionString;
        }

        public static string CreateConnectionString(string strHost, uint nPort, string strUsername, string strPassword, string strDomain)
        {
            SCConnectionStringBuilder connectionStringBuilder = new SCConnectionStringBuilder();

            connectionStringBuilder.ConnectionString = CreateBaseConnectionString(strHost, nPort);

            connectionStringBuilder.UserID = strUsername;
            connectionStringBuilder.Password = strPassword;
            connectionStringBuilder.WindowsDomain = strDomain;

            return connectionStringBuilder.ConnectionString;
        }

        public static string CreateConnectionString(string strHost, uint nPort)
        {
            return CreateBaseConnectionString(strHost, nPort);
        }

        public static string CreateConnectionString(string strHost)
        {
            string strStandardDeploymentPing = string.Empty;
            string strFarmDeploymentPing = string.Empty;
            int nStandardDeploymentDividerPosition = -1;
            int nFarmDeploymentDividerPosition = -1;
            uint nPort = 0;
            string strConnectionString = string.Empty;

            strStandardDeploymentPing = DiscoverHost.Ping(strHost, 49599, 1000);

            nStandardDeploymentDividerPosition = strStandardDeploymentPing.IndexOf("|");

            if (nStandardDeploymentDividerPosition >= 0)
            {
                nPort = Convert.ToUInt32(strStandardDeploymentPing.Substring(nStandardDeploymentDividerPosition + 1));

                strConnectionString = CreateBaseConnectionString(strHost, nPort);
            }
            else
            {
                strFarmDeploymentPing = DiscoverHost.Ping(strHost, 49600, 1000);

                nFarmDeploymentDividerPosition = strFarmDeploymentPing.IndexOf("|");
            
                if (nFarmDeploymentDividerPosition >= 0)
                {
                    nPort = Convert.ToUInt32(strFarmDeploymentPing.Substring(nFarmDeploymentDividerPosition + 1));

                    strConnectionString = CreateBaseConnectionString(strHost, nPort);
                }
                else
                {
                    throw new ArgumentException(string.Format(Resources.Exception_HostNotFound, new object[] { strHost }));
                }
            }

            return strConnectionString;
        }
    }
}

