using System;
using System.Collections.Generic;
using System.Text;

using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;


namespace Dauphine.Web.UI.SmartControls
{
    class SmartField
    {
        public static object GetCurrentValue(Control control)
        {
            IField field = null;
            WorkflowContextManager workflowContextManager = null;
            ClientContextManager clientContextManager = null;
            object value = string.Empty;

            field = (IField)control;

            if (field.Field.Length == 0)
            {
                throw new HttpException(string.Format(Resources.Exception_FieldIsBlank, new object[] { control.ID }));
            }
            else
            {
                workflowContextManager = Utility.FindWorkflowContextManager(control, field.WorkflowContextManagerID);
                
                if (field.Scope == FieldScope.Process)
                {
                    if (field.Type == FieldType.DataField)
                    {
                        if (workflowContextManager.ProcessInstanceDataFields[field.Field] == null)
                        {
                            throw new HttpException(string.Format(Resources.Exception_FieldNotFound, new object[] { field.Scope, field.Type, field.Field, control.ID }));
                        }
                        else
                        {
                            value = workflowContextManager.ProcessInstanceDataFields[field.Field];
                        }
                    }
                    else
                    {
                        if (workflowContextManager.ProcessInstanceXmlFields[field.Field] == null)
                        {
                            throw new HttpException(string.Format(Resources.Exception_FieldNotFound, new object[] { field.Scope, field.Type, field.Field, control.ID }));
                        }
                        else
                        {
                            value = workflowContextManager.ProcessInstanceXmlFields[field.Field];
                        }
                    }
                }
                else
                {
                    if (workflowContextManager is StartContextManager && field.Scope == FieldScope.Activity)
                    {
                        throw new NotSupportedException(Resources.Exception_ActivityFieldsNotSupported);
                    }
                    else
                    {
                        clientContextManager = (ClientContextManager)workflowContextManager;

                        if (field.Type == FieldType.DataField)
                        {
                            if (clientContextManager.ActivityInstanceDestinationDataFields[field.Field] == null)
                            {
                                throw new HttpException(string.Format(Resources.Exception_FieldNotFound, new object[] { field.Scope, field.Type, field.Field, control.ID }));
                            }
                            else
                            {
                                value = clientContextManager.ActivityInstanceDestinationDataFields[field.Field];
                            }
                        }
                        else
                        {
                            if (clientContextManager.ActivityInstanceDestinationXmlFields[field.Field] == null)
                            {
                                throw new HttpException(string.Format(Resources.Exception_FieldNotFound, new object[] { field.Scope, field.Type, field.Field, control.ID }));
                            }
                            else
                            {
                                value = clientContextManager.ActivityInstanceDestinationXmlFields[field.Field];
                            }
                        }
                    }
                }                
            }

            return value;            
        }

        private static List<Control> GetNonReadOnlyFields(Control control, List<Control> lstControls)
        {
            IField field = null;

            for (int i = 0; i < control.Controls.Count; i++)
            {
                if (control.Controls[i] is IField)
                {
                    field = (IField)control.Controls[i];

                    if(!field.LoadOnly)
                    {
                        lstControls.Add(control.Controls[i]);
                    }               
                }
                
                if (control.Controls[i].Controls.Count > 0)
                {
                    lstControls = GetNonReadOnlyFields(control.Controls[i], lstControls);
                }     
            }

            return lstControls;
        }

        public static List<Control> GetNonReadOnlyFields(WorkflowContextManager workflowContextManager)
        {
            List<Control> lstControls = new List<Control>();
            IField field = null;

            for (int i = 0; i < workflowContextManager.Page.Form.Controls.Count; i++)
            {
                if (workflowContextManager.Page.Form.Controls[i] is IField)
                {
                    field = (IField)workflowContextManager.Page.Form.Controls[i];

                    if (!field.LoadOnly)
                    {
                        lstControls.Add(workflowContextManager.Page.Form.Controls[i]);
                    }
                }

                if (workflowContextManager.Page.Form.Controls[i].Controls.Count > 0)
                {
                    lstControls = GetNonReadOnlyFields(workflowContextManager.Page.Form.Controls[i], lstControls);
                }   
            }

            return lstControls;
        }
    }
}
