using System;
using System.Collections.Generic;
using System.Text;

using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using System.Data;
using System.Resources;
using System.IO;

namespace Dauphine.Web.UI.SmartControls
{
    class Utility
    {
        private static Control RecursiveFindControl(Control control, string strControlID)
        {
            Control targetControl = null;

            for (int i = 0; i < control.Controls.Count && targetControl == null; i++)
            {
                if (control.Controls[i].ID == strControlID)
                {
                    targetControl = control.Controls[i];
                }
                else
                {
                    if (control.Controls[i].Controls.Count > 0)
                    {
                        targetControl = RecursiveFindControl(control.Controls[i], strControlID);
                    }
                }
            }

            return targetControl;
        }

        public static Control FindControl(Page page, string strControlID)
        {
            Control targetControl = null;

            for (int i = 0; i < page.Controls.Count && targetControl == null; i++)
            {
                if (page.Controls[i].ID == strControlID)
                {
                    targetControl = page.Controls[i];
                }
                else
                {
                    if (page.Controls[i].Controls.Count > 0)
                    {
                        targetControl = RecursiveFindControl(page.Controls[i], strControlID);
                    }
                }
            }

            return targetControl;
        }
        
        public static WorkflowContextManager FindWorkflowContextManager(Control controlCurrent, string strWorkflowContextManagerID)
        {
            Control control = null;
            WorkflowContextManager workflowContextManager = null;

            if (strWorkflowContextManagerID.Length != 0)
            {
                control = Utility.FindControl(controlCurrent.Page, strWorkflowContextManagerID);

                if (control == null)
                {
                    throw new HttpException(string.Format(Resources.Exception_WorkflowContextManagerNotFound, new object[] { strWorkflowContextManagerID, controlCurrent.ID }));
                }
                else
                {
                    if (control is WorkflowContextManager)
                    {
                        workflowContextManager = (WorkflowContextManager)control;
                    }
                    else
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_InvalidWorkflowContextManager, new object[] { strWorkflowContextManagerID, controlCurrent.ID }));
                    }
                }
            }
            else
            {
                throw new HttpException(string.Format(Resources.Exception_WorkflowContextManagerIsBlank, new object[] { controlCurrent.ID }));
            }

            return workflowContextManager;
        }

        public static StartContextManager FindStartContextManager(Control controlCurrent, string strStartContextManagerID)
        {
            Control control = null;
            StartContextManager startContextManager = null;

            if (strStartContextManagerID.Length != 0)
            {
                control = Utility.FindControl(controlCurrent.Page, strStartContextManagerID);

                if (control == null)
                {
                    throw new HttpException(string.Format(Resources.Exception_StartContextManagerNotFound, strStartContextManagerID, controlCurrent.ID));
                }
                else
                {
                    if (control is StartContextManager)
                    {
                        startContextManager = (StartContextManager)control;
                    }
                    else
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_InvalidStartContextManager, new object[] { strStartContextManagerID, controlCurrent.ID }));
                    }
                }
            }
            else
            {
                throw new HttpException(string.Format(Resources.Exception_StartContextManagerIsBlank, new object[] { controlCurrent.ID }));
            }

            return startContextManager;
        }

        public static ClientContextManager FindClientContextManager(Control controlCurrent, string strClientContextManagerID)
        {
            Control control = null;
            ClientContextManager clientContextManager = null;

            if (strClientContextManagerID.Length != 0)
            {
                control = Utility.FindControl(controlCurrent.Page, strClientContextManagerID);

                if (control == null)
                {
                    throw new HttpException(string.Format(Resources.Exception_ClientContextManagerNotFound, new object[] { strClientContextManagerID, controlCurrent.ID }));
                }
                else
                {
                    if (control is ClientContextManager)
                    {
                        clientContextManager = (ClientContextManager)control;
                    }
                    else
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_InvalidClientContextManager, new object[] { strClientContextManagerID, controlCurrent.ID }));
                    }
                }
            }
            else
            {
                throw new HttpException(string.Format(Resources.Exception_ClientContextManagerIsBlank, new object[] { controlCurrent.ID }));
            }

            return clientContextManager;
        }

        public static IActionList FindActionsListControl(Control controlCurrent, string strActionsListControlID)
        {
            Control control = null;
            IActionList actionsListControl = null;

            if (strActionsListControlID.Length != 0)
            {
                control = Utility.FindControl(controlCurrent.Page, strActionsListControlID);

                if (control == null)
                {
                    throw new HttpException(string.Format(Resources.Exception_ActionsListControlNotFound, new object[] { strActionsListControlID, controlCurrent.ID }));
                }
                else
                {
                    if ((control is IActionList) && (control is ListControl))
                    {
                        actionsListControl = (IActionList)control;
                    }
                    else
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_InvalidActionsListControl, new object[] { strActionsListControlID, control.ID }));
                    }
                }
            }
            else
            {
                throw new HttpException(string.Format(Resources.Exception_ActionsListControlIsBlank, new object[] { controlCurrent.ID }));
            }

            return actionsListControl;
        }
    }
}
