using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using SourceCode.Workflow.Management;

namespace Dauphine.Web.UI.SmartControls
{
    class WorkflowServerManager : WorkflowManagementServer
    {
        private string _connectionString = string.Empty;

        public string ConnectionString
        {
            get
            {
                return _connectionString;
            }
        }

        public WorkflowServerManager(string strHost, uint nPort)
        {
            _connectionString = ConnectionStringBuilder.CreateConnectionString(strHost, nPort);
        }

        public WorkflowServerManager(string strHost)
        {
            _connectionString = ConnectionStringBuilder.CreateConnectionString(strHost);
        }

        public void OpenConnection()
        {
            if (this.Connection == null)
            {
                this.Connection = this.CreateConnection();
            }

            this.Connection.Open(_connectionString);

            this.GetServerData();
        }

        public void CloseConnection()
        {
            if (this.Connection != null)
            {
                this.Connection.Close();
            }

            this.DeleteConnection();
        }

        public ProcessFolder GetProcessFolder(string strProcessFolderName)
        {
            ProcessFolders processFolders = null;
            ProcessFolder processFolder = null;

            try
            {
                processFolders = this.GetProcessFolders();

                try
                {
                    processFolder = processFolders[strProcessFolderName];
                }
                catch
                {
                    throw new ArgumentException(string.Format(Resources.Exception_ProcessFolderNotFound, new object[] { strProcessFolderName }));
                }
            }
            catch
            {
                throw;
            }

            return processFolder;
        }

        public ProcessSet GetProcessSet(string strProcessSetFullName)
        {            
            int nPosition = -1;
            string strProcessFolderName = string.Empty;
            string strProcessSetName = string.Empty;
            ProcessFolder processFolder = null;
            ProcessSet processSet = null;

            try
            {
                nPosition = strProcessSetFullName.IndexOf(Path.DirectorySeparatorChar);
                
                if (nPosition >= 0)
                {
                    strProcessFolderName = strProcessSetFullName.Substring(0, nPosition);
                    strProcessSetName = strProcessSetFullName.Substring(nPosition + 1);

                    processFolder = this.GetProcessFolder(strProcessFolderName);

                    try
                    {
                        processSet = processFolder.ProcessSets[strProcessSetName];
                    }
                    catch
                    {
                        throw new ArgumentException(string.Format(Resources.Exception_ProcessNotFound, new object[] { strProcessSetName }));
                    }
                }
                else
                {
                    throw new FormatException(string.Format(Resources.Exception_IncorrectProcessFormat, new object[] { strProcessSetFullName }));
                }
            }
            catch
            {
                throw;
            }

            return processSet;
        }

        public Activity GetActivity(string strProcessFullName, string strActivityName)
        {
            ProcessSet processSet = null;
            Activities activities = null;
            Activity activity = null;

            try
            {
                processSet = this.GetProcessSet(strProcessFullName);

                activities = this.GetProcActivities(processSet.ProcID);

                try
                {
                    activity = activities[strActivityName];
                }
                catch
                {
                    throw new ArgumentException(string.Format(Resources.Exception_ActivityNotFound, new object[] { strActivityName }));
                }
            }
            catch
            {                
                throw;
            }

            return activity;
        } 
    }
}

