using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Drawing;
using System.Drawing.Design;
using System.Web.UI.Design;

using Dauphine.Web.UI.SmartControls.Design;

namespace Dauphine.Web.UI.SmartControls
{
    [DefaultProperty("StartContextManagerID")]
    [Designer(typeof(StartButtonDesigner))]
    [ToolboxBitmap(typeof(StartButton), "Resources.ToolboxBitmaps.StartButton.bmp")]
    [ToolboxData(@"<{0}:StartButton runat=""server"" Text=""StartButton""></{0}:StartButton>")]
    public class StartButton : Button, IStartButton
    {
        private const string DefaultValue_StartContextManagerID = "";
        private const string DefaultValue_RedirectUrl = "";

        [DefaultValue(DefaultValue_StartContextManagerID)]
        [SmartDescription("DescriptionAttribute_StartButton_StartContextManagerID")]
        [SmartCategory("CategoryAttribute_StartButton_StartContextManagerID")]
        [IDReferenceProperty(typeof(StartContextManager))]
        [TypeConverter(typeof(StartContextManagerConverter))]
        public string StartContextManagerID
        {
            get
            {
                Object oObj = ViewState["StartContextManagerID"];

                if (oObj == null)
                {
                    ViewState["StartContextManagerID"] = DefaultValue_StartContextManagerID;
                }

                return ViewState["StartContextManagerID"] + string.Empty;
            }

            set
            {
                ViewState["StartContextManagerID"] = value;
            }
        }

        [DefaultValue(DefaultValue_RedirectUrl)]
        [SmartDescription("DescriptionAttribute_StartButton_RedirectUrl")]
        [SmartCategory("CategoryAttribute_StartButton_RedirectUrl")]
        [Editor(typeof(UrlEditor), typeof(UITypeEditor))]
        public string RedirectUrl
        {
            get
            {
                Object oObj = ViewState["RedirectUrl"];

                if (oObj == null)
                {
                    ViewState["RedirectUrl"] = DefaultValue_RedirectUrl;
                }

                return ViewState["RedirectUrl"] + string.Empty;
            }

            set
            {
                ViewState["RedirectUrl"] = value;
            }
        }

        protected override void OnClick(EventArgs e)
        {
            StartContextManager startContextManager = null;

            try
            {
                base.OnClick(e);

                startContextManager = Utility.FindStartContextManager(this, this.StartContextManagerID);

                startContextManager.StartProcessInstance();

                if (this.RedirectUrl.Length != 0)
                {
                    this.Context.Response.Redirect(this.RedirectUrl);
                }
            }
            catch
            {
                throw;
            }
        }
    }
}
